﻿using System;
using System.Drawing;
using System.Configuration;
using DevExpress.XtraTreeList;

using DotSpatial.Controls;
using DotSpatial.Data;
using DotSpatial.Symbology;
using DotSpatial.Plugins.OverviewMap.Properties;
using DotSpatial.Controls.Docking;
using System.Windows.Forms;
using GeoAPI.Geometries;
using DotSpatial.NTSExtension;

namespace DotSpatial.Plugins.OverviewMap
{
    public class OverviewMapPlugin : Extension
    {
        public OverviewMapPlugin()
        {
            DeactivationAllowed = false;
        }

        private OverviewMapControl OverviewMapControl { get; set; }
        private TreeList TreeList { get; set; }
        private IMapFeatureLayer MapAreaLayer { get; set; }
        private IMapFeatureLayer OverviewMapLayer { get; set; }
        private MoveMapAreaFunction MoveMapAreaFunction { get; set; }
        private IMap OverviewMap 
        { 
            get
            {
                return (IMap)((Map)this.App.Map).Tag;
            }
            set
            {
                ((Map)this.App.Map).Tag = value;
            }
        }
        

        #region Public Methods

        /// <summary>
        /// Initialize the DotSpatial plugin
        /// </summary>
        public override void Activate()
        {
            this.App.Map.MapFrame.ViewExtentsChanged += OnMapViewExtentsChanged;
            //        this.App.Map.MapFrame.Layers.ItemChanged += OnItemChanged;

            this.OverviewMapControl = new OverviewMapControl();
            this.TreeList = this.OverviewMapControl.TreeList;

            this.OverviewMap = this.OverviewMapControl.OverviewMap;
            this.OverviewMap.MapFunctions.Clear();
            this.OverviewMap.FunctionMode = FunctionMode.None;
            this.OverviewMap.Projection = this.App.Map.Projection;
            this.OverviewMap.MapFrame.ProjectionModeDefine = ActionMode.Always;

            this.MoveMapAreaFunction = new MoveMapAreaFunction(this.OverviewMap);
            this.MoveMapAreaFunction.MainMap = this.App.Map;

            this.OverviewMap.MapFunctions.Add(this.MoveMapAreaFunction);
            App.DockManager.Add(new DockablePanel("kOvMap", "索引図", this.OverviewMapControl, DockStyle.Right) { SmallImage = Resources.map_16x16 });


            var imageList = new ImageList();
            imageList.Images.Add(Resources.Action_New);
            imageList.Images.Add(Resources.Action_Open);
            this.TreeList.SelectImageList = imageList;

            var xml = ConfigurationManager.AppSettings["address_tree.file"];
            this.TreeList.ImportFromXml(xml);
            this.TreeList.Columns[1].Visible = false;
            this.TreeList.Columns[2].Visible = false;
            this.TreeList.MouseClick += new System.Windows.Forms.MouseEventHandler(this.OnMouseClickTreeNode);

            this.AddOverViewMap();
            base.Activate();
        }

        private void OnMouseClickTreeNode(object sender, EventArgs e)
        {
            var pt = this.TreeList.PointToClient(Cursor.Position);
            var info = this.TreeList.CalcHitInfo(pt);
            if (info.Node != null)
            {
                var x = info.Node.GetValue(1);
                var y = info.Node.GetValue(2);
                


            }

        }

        /// <summary>
        /// Fires when the plugin should become inactive
        /// </summary>
        public override void Deactivate()
        {
            this.App.Map.MapFrame.ViewExtentsChanged -= OnMapViewExtentsChanged;

            this.App.HeaderControl.RemoveAll();
            this.App.DockManager.Remove("kOvMap");

            base.Deactivate();
        }

        private void AddOverViewMap()
        {
            var fileName = ConfigurationManager.AppSettings["overview_map.file"];

            var provider = new ShapefileDataProvider();
            var featureSet = provider.Open(fileName);
            featureSet.Projection = this.App.Map.Projection;
            this.OverviewMapLayer = (IMapFeatureLayer)this.OverviewMap.Layers.Add(featureSet);
            ((PolygonSymbolizer)this.OverviewMapLayer.Symbolizer).SetFillColor(
                ColorTranslator.FromHtml(ConfigurationManager.AppSettings["overview_map.fill_color"]));
            ((PolygonSymbolizer)this.OverviewMapLayer.Symbolizer).SetOutline(
                ColorTranslator.FromHtml(ConfigurationManager.AppSettings["overview_map.out_line_color"]),
                double.Parse(ConfigurationManager.AppSettings["overview_map.out_line_width"]));
            
            this.MapAreaLayer = this.OverviewMap.Layers.Add(new FeatureSet(FeatureType.Line));
            ((LineSymbolizer)this.MapAreaLayer.Symbolizer).SetFillColor(Color.Red);
            ((LineSymbolizer)this.MapAreaLayer.Symbolizer).SetWidth(2);

            var linearRing = featureSet.Extent.ToEnvelope().ToLinearRing();
            this.MapAreaLayer.DataSet.AddFeature(linearRing);

            this.MoveMapAreaFunction.ActiveLayer = this.MapAreaLayer;
            this.MoveMapAreaFunction.Activate();

            this.OverviewMap.ZoomToMaxExtent();

            this.OverviewMap.Resized += OnResized;
        }

        private void OnResized(object sender, EventArgs args)
        {
            //this.MapAreaLayer.DataSet.Features.Clear();
            //var linearRing = this.App.Map.Extent.ToEnvelope().ToLinearRing();
            //this.MapAreaLayer.DataSet.AddFeature(linearRing);

            const double eps = 1e-7;
            Envelope layerEnvelope = this.OverviewMapLayer.Extent.ToEnvelope();
            if (layerEnvelope.Width > eps && layerEnvelope.Height > eps)
            {
                layerEnvelope.ExpandBy(layerEnvelope.Width / 10, layerEnvelope.Height / 10); // work item #84
            }
            else
            {
                double zoomInFactor = 0.05; //fixed zoom-in by 10% - 5% on each side
                double newExtentWidth = this.OverviewMap.ViewExtents.Width * zoomInFactor;
                double newExtentHeight = this.OverviewMap.ViewExtents.Height * zoomInFactor;
                layerEnvelope.ExpandBy(newExtentWidth, newExtentHeight);
            }

            this.OverviewMap.ViewExtents = layerEnvelope.ToExtent();
        }

        private void OnMapViewExtentsChanged(object sender, ExtentArgs args)
        {
            if (this.OverviewMap.Layers.Count == 0 || this.MapAreaLayer == null)
            {
                return;
            }

            this.MapAreaLayer.DataSet.Features.Clear();
            var linearRing = args.Extent.ToEnvelope().ToLinearRing();
            this.MapAreaLayer.DataSet.AddFeature(linearRing);

            if (!this.OverviewMap.IsZoomedToMaxExtent)
            {
                var center = linearRing.Centroid.Coordinate;
                this.OverviewMap.ViewExtents.SetCenter(linearRing.Centroid.Coordinate);
            }

            this.OverviewMap.MapFrame.Invalidate();
        }



        #endregion
    }
}
